VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsACM"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Type ACM_StreamHeader
    hdr             As ACMSTREAMHEADER
    RealInLen       As Long
    RealOutLen      As Long
End Type

Private Type ACMStream
    hStream         As Long
    hDriver         As Long
    haid            As Long
    headers()       As ACM_StreamHeader
    headercount     As Integer
End Type
Private stream      As ACMStream

Public Sub CloseStream()
    Dim i   As Integer
    Dim mmr As Long

    ' reset whole stream
    mmr = acmStreamReset(stream.hStream, 0)
    If mmr <> MMSYSERR_NOERROR Then
        Debug.Print "CloseStream: acmStreamReset Error"
    End If

    ' remove all stream headers
    For i = 0 To stream.headercount - 1
        With stream.headers(i)
            .hdr.cbSrcLength = .RealInLen
            .hdr.cbDstLength = .RealOutLen
        End With
        mmr = acmStreamUnprepareHeader(stream.hStream, stream.headers(i).hdr, 0)
        If mmr <> MMSYSERR_NOERROR Then
            Debug.Print "CloseStream: acmStreamUnprepare Header - Error bei " & i
        End If
    Next
    stream.headercount = 0

    ' close stream
    mmr = acmStreamClose(stream.hStream, 0)
    If mmr <> MMSYSERR_NOERROR Then
        Debug.Print "CloseStream: acmStreamClose Error"
    End If

    ' if we used a driver, close it
    If stream.hDriver <> 0 Then
        mmr = acmDriverClose(stream.hDriver, 0)
        If mmr <> MMSYSERR_NOERROR Then
            Debug.Print "CloseStream: acmDriverClose Error"
        End If
    End If
End Sub

Public Function CreateStream(ByVal haid As Long, WFXIn() As Byte, WFXOut() As Byte, Optional options As ACM_STREAMOPENF) As Boolean
    Dim hDriver As Long
    Dim hStream As Long
    Dim mmr     As Long

    ' if driver ID exists, get its handle
    If haid <> 0 Then
        mmr = acmDriverOpen(hDriver, haid, 0)
        If mmr <> 0 Then Exit Function
    End If

    ' create the stream
    mmr = acmStreamOpen(hStream, hDriver, WFXIn(0), WFXOut(0), 0, 0, 0, options)
    If mmr <> 0 Then
        acmDriverClose hDriver, 0
        Exit Function
    End If

    ' save stream/driver handle
    With stream
        .haid = haid
        .hDriver = hDriver
        .hStream = hStream
    End With

    CreateStream = True
End Function

Public Function StreamOutputSize(InpLen As Long) As Long
    Dim mmr     As Long
    Dim OutLen  As Long

    ' suggested output buffer size for the stream
    mmr = acmStreamSize(stream.hStream, InpLen, OutLen, ACM_STREAMSIZEF_SOURCE)
    If mmr <> MMSYSERR_NOERROR Then Exit Function

    StreamOutputSize = OutLen
End Function

Public Function AddStreamHeader(pInput As Long, InpLen As Long, pOutput As Long, OutLen As Long) As Long
    Dim hdr As ACMSTREAMHEADER
    Dim mmr As Long

    ' Stream Header
    With hdr
        .cbStruct = Len(hdr)
        .pbSrc = pInput
        .cbSrcLength = InpLen
        .pbDst = pOutput
        .cbDstLength = OutLen
    End With

    ' save stream header
    With stream
        ReDim Preserve .headers(.headercount)
        With .headers(.headercount)
            .hdr = hdr
            .RealInLen = InpLen     ' important for closing the stream
            .RealOutLen = OutLen
        End With
    End With

    With stream
        ' add stream header to the stream
        mmr = acmStreamPrepareHeader(.hStream, .headers(.headercount).hdr, 0)
        If mmr = MMSYSERR_NOERROR Then
            AddStreamHeader = stream.headercount
            stream.headercount = stream.headercount + 1
        Else
            AddStreamHeader = -1
        End If
    End With
End Function

Public Function ConvertData(streamheader As Integer, btOut() As Byte, Optional InLen As Long, Optional IsLastConv As Boolean) As Long
    Dim mmr         As Long
    Dim flags       As Long

    ' new InLen?
    With stream.headers(streamheader)
        If InLen > 0 Then
            .hdr.cbSrcLength = InLen
        Else
            .hdr.cbSrcLength = .RealInLen
        End If
    End With

    ' convert data
    With stream
        mmr = acmStreamConvert(.hStream, .headers(streamheader).hdr, ACM_STREAMCONVERTF_BLOCKALIGN)
    End With

    If mmr <> MMSYSERR_NOERROR Then
        ConvertData = -1
        Exit Function
    End If

    With stream
        With .headers(streamheader)

            ' something got converted?
            If .hdr.cbDstLengthUsed = 0 Then
                ConvertData = 0
                Exit Function
            End If

            ' return new output buffer for converted data
            ReDim btOut(.hdr.cbDstLengthUsed - 1) As Byte
            CopyMemory btOut(0), .hdr.pbDst, .hdr.cbDstLengthUsed

            Dim lngLast As Long
            lngLast = .hdr.cbDstLengthUsed

            ' if this was the last packet, get last data from the codec
            If IsLastConv Then
                .hdr.cbSrcLength = 0
                mmr = acmStreamConvert(stream.hStream, .hdr, ACM_STREAMCONVERTF_END)

                ' redimension output buffer another time
                ' to push the last data into it
               If .hdr.cbDstLengthUsed > 0 Then
                    ReDim Preserve btOut(lngLast + .hdr.cbDstLengthUsed - 1) As Byte
                    CopyMemory btOut(lngLast), ByVal .hdr.pbDst, .hdr.cbDstLengthUsed
                End If
            End If

            ConvertData = 1
        End With
    End With
End Function
